@@ -26,8 +26,7 @@
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#ifndef AccessibilityRenderObject_h
-#define AccessibilityRenderObject_h
+#pragma once
 
 #include "AccessibilityNodeObject.h"
 #include "LayoutRect.h"
@@ -38,19 +37,14 @@ namespace WebCore {
 class AccessibilitySVGRoot;
 class AXObjectCache;
 class Element;
-class Frame;
 class FrameView;
-class HitTestResult;
-class HTMLAnchorElement;
 class HTMLAreaElement;
 class HTMLElement;
 class HTMLLabelElement;
 class HTMLMapElement;
-class HTMLSelectElement;
 class IntPoint;
 class IntSize;
 class Node;
-class RenderListBox;
 class RenderTextControl;
 class RenderView;
 class VisibleSelection;
@@ -158,6 +152,7 @@ public:
     void textChanged() override;
     void addChildren() override;
     bool canHaveChildren() const override;
+    bool canHaveSelectedChildren() const override;
     void selectedChildren(AccessibilityChildrenVector&) override;
     void visibleChildren(AccessibilityChildrenVector&) override;
     void tabChildren(AccessibilityChildrenVector&) override;
@@ -279,7 +274,7 @@ private:
     const String ariaLiveRegionStatus() const override;
     const AtomicString& ariaLiveRegionRelevant() const override;
     bool ariaLiveRegionAtomic() const override;
-    bool ariaLiveRegionBusy() const override;
+    bool isBusy() const override;
 
     bool inheritsPresentationalRole() const override;
 
@@ -289,5 +284,3 @@ private:
 } // namespace WebCore
 
 SPECIALIZE_TYPE_TRAITS_ACCESSIBILITY(AccessibilityRenderObject, isAccessibilityRenderObject())
-
-#endif // AccessibilityRenderObject_h
