@@ -109,6 +109,11 @@ void RenderGrid::addChild(RenderObject* newChild, RenderObject* beforeChild)
 {
     RenderBlock::addChild(newChild, beforeChild);
 
+    // Positioned grid items do not take up space or otherwise participate in the layout of the grid,
+    // for that reason we don't need to mark the grid as dirty when they are added.
+    if (newChild->isOutOfFlowPositioned())
+        return;
+
     // The grid needs to be recomputed as it might contain auto-placed items that
     // will change their position.
     dirtyGrid();
@@ -118,6 +123,11 @@ void RenderGrid::removeChild(RenderObject& child)
 {
     RenderBlock::removeChild(child);
 
+    // Positioned grid items do not take up space or otherwise participate in the layout of the grid,
+    // for that reason we don't need to mark the grid as dirty when they are removed.
+    if (child.isOutOfFlowPositioned())
+        return;
+
     // The grid needs to be recomputed as it might contain auto-placed items that
     // will change their position.
     dirtyGrid();
