@@ -38,9 +38,16 @@ RenderTextControlMultiLine::RenderTextControlMultiLine(HTMLTextAreaElement& elem
 }
 
 RenderTextControlMultiLine::~RenderTextControlMultiLine()
+{
+    // Do not add any code here. Add it to willBeDestroyed() instead.
+}
+
+void RenderTextControlMultiLine::willBeDestroyed()
 {
     if (textAreaElement().isConnected())
         textAreaElement().rendererWillBeDestroyed();
+
+    RenderTextControl::willBeDestroyed();
 }
 
 HTMLTextAreaElement& RenderTextControlMultiLine::textAreaElement() const
