@@ -20,16 +20,13 @@
  *
  */
 
-#ifndef RenderTextControlSingleLine_h
-#define RenderTextControlSingleLine_h
+#pragma once
 
 #include "HTMLInputElement.h"
 #include "RenderTextControl.h"
 
 namespace WebCore {
 
-class HTMLInputElement;
-
 class RenderTextControlSingleLine : public RenderTextControl {
 public:
     RenderTextControlSingleLine(HTMLInputElement&, RenderStyle&&);
@@ -105,5 +102,3 @@ private:
 
 SPECIALIZE_TYPE_TRAITS_RENDER_OBJECT(RenderTextControlSingleLine, isTextField())
 SPECIALIZE_TYPE_TRAITS_RENDER_OBJECT(RenderTextControlInnerBlock, isTextControlInnerBlock())
-
-#endif // RenderTextControlSingleLine_h
