@@ -20,18 +20,15 @@
  *
  */
 
-#ifndef XSLStyleSheet_h
-#define XSLStyleSheet_h
+#pragma once
 
 #if ENABLE(XSLT)
 
 #include "ProcessingInstruction.h"
 #include "StyleSheet.h"
-
 #include <libxml/parser.h>
 #include <libxslt/transform.h>
-
-#include <wtf/PassRefPtr.h>
+#include <wtf/Ref.h>
 #include <wtf/TypeCasts.h>
 
 namespace WebCore {
@@ -41,24 +38,24 @@ class XSLImportRule;
     
 class XSLStyleSheet final : public StyleSheet {
 public:
-    static PassRefPtr<XSLStyleSheet> create(XSLImportRule* parentImport, const String& originalURL, const URL& finalURL)
+    static Ref<XSLStyleSheet> create(XSLImportRule* parentImport, const String& originalURL, const URL& finalURL)
     {
-        return adoptRef(new XSLStyleSheet(parentImport, originalURL, finalURL));
+        return adoptRef(*new XSLStyleSheet(parentImport, originalURL, finalURL));
     }
-    static PassRefPtr<XSLStyleSheet> create(ProcessingInstruction* parentNode, const String& originalURL, const URL& finalURL)
+    static Ref<XSLStyleSheet> create(ProcessingInstruction* parentNode, const String& originalURL, const URL& finalURL)
     {
-        return adoptRef(new XSLStyleSheet(parentNode, originalURL, finalURL, false));
+        return adoptRef(*new XSLStyleSheet(parentNode, originalURL, finalURL, false));
     }
-    static PassRefPtr<XSLStyleSheet> createEmbedded(ProcessingInstruction* parentNode, const URL& finalURL)
+    static Ref<XSLStyleSheet> createEmbedded(ProcessingInstruction* parentNode, const URL& finalURL)
     {
-        return adoptRef(new XSLStyleSheet(parentNode, finalURL.string(), finalURL, true));
+        return adoptRef(*new XSLStyleSheet(parentNode, finalURL.string(), finalURL, true));
     }
 
     // Taking an arbitrary node is unsafe, because owner node pointer can become stale.
     // XSLTProcessor ensures that the stylesheet doesn't outlive its parent, in part by not exposing it to JavaScript.
-    static PassRefPtr<XSLStyleSheet> createForXSLTProcessor(Node* parentNode, const String& originalURL, const URL& finalURL)
+    static Ref<XSLStyleSheet> createForXSLTProcessor(Node* parentNode, const String& originalURL, const URL& finalURL)
     {
-        return adoptRef(new XSLStyleSheet(parentNode, originalURL, finalURL, false));
+        return adoptRef(*new XSLStyleSheet(parentNode, originalURL, finalURL, false));
     }
 
     virtual ~XSLStyleSheet();
@@ -75,7 +72,7 @@ public:
     CachedResourceLoader* cachedResourceLoader();
 
     Document* ownerDocument();
-    virtual XSLStyleSheet* parentStyleSheet() const override { return m_parentStyleSheet; }
+    XSLStyleSheet* parentStyleSheet() const override { return m_parentStyleSheet; }
     void setParentStyleSheet(XSLStyleSheet* parent);
 
     xmlDocPtr document();
@@ -87,22 +84,22 @@ public:
     void markAsProcessed();
     bool processed() const { return m_processed; }
     
-    virtual String type() const override { return "text/xml"; }
-    virtual bool disabled() const override { return m_isDisabled; }
-    virtual void setDisabled(bool b) override { m_isDisabled = b; }
-    virtual Node* ownerNode() const override { return m_ownerNode; }
-    virtual String href() const override { return m_originalURL; }
-    virtual String title() const override { return emptyString(); }
+    String type() const override { return "text/xml"; }
+    bool disabled() const override { return m_isDisabled; }
+    void setDisabled(bool b) override { m_isDisabled = b; }
+    Node* ownerNode() const override { return m_ownerNode; }
+    String href() const override { return m_originalURL; }
+    String title() const override { return emptyString(); }
 
-    virtual void clearOwnerNode() override { m_ownerNode = 0; }
-    virtual URL baseURL() const override { return m_finalURL; }
-    virtual bool isLoading() const override;
+    void clearOwnerNode() override { m_ownerNode = nullptr; }
+    URL baseURL() const override { return m_finalURL; }
+    bool isLoading() const override;
 
 private:
     XSLStyleSheet(Node* parentNode, const String& originalURL, const URL& finalURL, bool embedded);
     XSLStyleSheet(XSLImportRule* parentImport, const String& originalURL, const URL& finalURL);
 
-    virtual bool isXSLStyleSheet() const override { return true; }
+    bool isXSLStyleSheet() const override { return true; }
     
     Node* m_ownerNode;
     String m_originalURL;
@@ -116,7 +113,8 @@ private:
 
     xmlDocPtr m_stylesheetDoc;
     bool m_stylesheetDocTaken;
-    
+    bool m_compilationFailed { false };
+
     XSLStyleSheet* m_parentStyleSheet;
 };
 
@@ -127,5 +125,3 @@ SPECIALIZE_TYPE_TRAITS_BEGIN(WebCore::XSLStyleSheet)
 SPECIALIZE_TYPE_TRAITS_END()
 
 #endif // ENABLE(XSLT)
-
-#endif // XSLStyleSheet_h
