@@ -31,6 +31,7 @@ WebInspector.TimelineRecordBar = class TimelineRecordBar extends WebInspector.Ob
 
         this._element = document.createElement("div");
         this._element.classList.add("timeline-record-bar");
+        this._element[WebInspector.TimelineRecordBar.ElementReferenceSymbol] = this;
 
         this.renderMode = renderMode;
         this.records = records;
@@ -166,6 +167,11 @@ WebInspector.TimelineRecordBar = class TimelineRecordBar extends WebInspector.Ob
             createBarCallback(activeRecords, WebInspector.TimelineRecordBar.RenderMode.ActiveOnly);
     }
 
+    static fromElement(element)
+    {
+        return element[WebInspector.TimelineRecordBar.ElementReferenceSymbol] || null;
+    }
+
     // Public
 
     get element()
@@ -362,6 +368,8 @@ WebInspector.TimelineRecordBar = class TimelineRecordBar extends WebInspector.Ob
     }
 };
 
+WebInspector.TimelineRecordBar.ElementReferenceSymbol = Symbol("timeline-record-bar");
+
 WebInspector.TimelineRecordBar.MinimumWidthPixels = 4;
 WebInspector.TimelineRecordBar.MinimumMarginPixels = 1;
 
