@@ -41,18 +41,16 @@
 
 namespace WebKit {
 
-void InitializeWebKit2(ProcessType processType)
+void InitializeWebKit2()
 {
 #if PLATFORM(COCOA)
     InitWebCoreSystemInterface();
 #endif
-    platformInitializeWebKit2(processType);
 #if PLATFORM(IOS)
     InitWebCoreThreadSystemInterface();
 #endif
 
     JSC::initializeThreading();
-    WTF::initializeMainThread();
     RunLoop::initializeMainRunLoop();
 
 #if !LOG_DISABLED || !RELEASE_LOG_DISABLED
