@@ -40,7 +40,7 @@ class WebFrame;
 
 class InjectedBundleHitTestResult : public API::ObjectImpl<API::Object::Type::BundleHitTestResult> {
 public:
-    static PassRefPtr<InjectedBundleHitTestResult> create(const WebCore::HitTestResult&);
+    static Ref<InjectedBundleHitTestResult> create(const WebCore::HitTestResult&);
 
     const WebCore::HitTestResult& coreHitTestResult() const { return m_hitTestResult; }
 
@@ -60,6 +60,7 @@ public:
 
     String linkLabel() const;
     String linkTitle() const;
+    String linkSuggestedFilename() const;
     
     WebCore::IntRect imageRect() const;
     
