@@ -25,11 +25,10 @@
  * THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#ifndef WebFrameNetworkingContext_h
-#define WebFrameNetworkingContext_h
+#pragma once
 
-#include "HTTPCookieAcceptPolicy.h"
 #include <WebCore/FrameNetworkingContext.h>
+#include <WebCore/SessionID.h>
 
 namespace WebKit {
 
@@ -38,22 +37,19 @@ class WebFrameLoaderClient;
 
 class WebFrameNetworkingContext : public WebCore::FrameNetworkingContext {
 public:
-    static PassRefPtr<WebFrameNetworkingContext> create(WebFrame* frame)
+    static Ref<WebFrameNetworkingContext> create(WebFrame* frame)
     {
-        return adoptRef(new WebFrameNetworkingContext(frame));
+        return adoptRef(*new WebFrameNetworkingContext(frame));
     }
 
-    static void ensurePrivateBrowsingSession(uint64_t sessionID);
-    static void setCookieAcceptPolicyForAllContexts(HTTPCookieAcceptPolicy);
+    static void ensurePrivateBrowsingSession(WebCore::SessionID);
 
     WebFrameLoaderClient* webFrameLoaderClient() const;
 
 private:
     WebFrameNetworkingContext(WebFrame*);
 
-    virtual WebCore::NetworkStorageSession& storageSession() const;
+    WebCore::NetworkStorageSession& storageSession() const override;
 };
 
 }
-
-#endif // WebFrameNetworkingContext_h
