@@ -32,6 +32,7 @@
 namespace bmalloc {
 
 class Deallocator;
+class DebugHeap;
 class Heap;
 
 // Per-cache object allocator.
@@ -64,7 +65,7 @@ private:
     std::array<BumpAllocator, sizeClassCount> m_bumpAllocators;
     std::array<BumpRangeCache, sizeClassCount> m_bumpRangeCaches;
 
-    bool m_isBmallocEnabled;
+    DebugHeap* m_debugHeap;
     Deallocator& m_deallocator;
 };
 
